#! /usr/bin/env python
# -*- python -*-

# @HEADER
# ************************************************************************
#
#                PyTrilinos: Python Interface to Trilinos
#                   Copyright (2005) Sandia Corporation
#
# Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
# license for use of this work by or on behalf of the U.S. Government.
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# Questions? Contact Bill Spotz (wfspotz@sandia.gov)
#
# ************************************************************************
# @HEADER

#
# System imports
from   numpy    import *
from   optparse import *
import sys
import unittest

#
# Parse the command-line arguments
parser = OptionParser()
parser.add_option("-b", "--use-boost", action="store_true", dest="boost",
                  default=False,
                  help="test the experimental boost-generated PyTrilinos package")
parser.add_option("-t", "--testharness", action="store_true",
                  dest="testharness", default=False,
                  help="test local build modules; prevent loading system-installed modules")
parser.add_option("-v", "--verbosity", type="int", dest="verbosity", default=2,
                  help="set the verbosity level [default 2]")
options,args = parser.parse_args()

#
# Under normal usage, simply use 'from PyTrilinos import Epetra'.  For testing,
# we want to be able to control whether we import from the build directory or
# from a system-installed version of PyTrilinos.
from testutil import fromPyTrilinosImport
Epetra    = fromPyTrilinosImport('Epetra'   , options.testharness)
EpetraExt = fromPyTrilinosImport('EpetraExt', options.testharness)

####################################################################

class ColoringTestCase(unittest.TestCase):
    "TestCase class for EpetraExt coloring objects"

    def setUp(self):
        self.comm = Epetra.PyComm()
        self.size = 9 * self.comm.NumProc()
        self.map  = Epetra.Map(self.size,0,self.comm)
        self.crsg = Epetra.CrsGraph(Epetra.Copy, self.map, 3)
        n         = self.size
        for lrid in range(self.crsg.NumMyRows()):
            grid = self.crsg.GRID(lrid)
            if   grid == 0  : indices = [0,1]
            elif grid == n-1: indices = [n-2,n-1]
            else            : indices = [grid-1,grid,grid+1]
            self.crsg.InsertGlobalIndices(grid,indices)
        self.crsg.FillComplete()

    def testMapColoring(self):
        "Test EpetraExt CrsGraph-to-MapColoring transform"
        mapColoring  = EpetraExt.CrsGraph_MapColoring(False)
        colorMap     = mapColoring(self.crsg)
        #for p in range(self.comm.NumProc()):
        #    if p == self.comm.MyPID(): print colorMap
        #    self.comm.Barrier()
        numColors    = colorMap.NumColors()
        defaultColor = colorMap.DefaultColor()
        self.assertEqual(numColors   , 3)
        self.assertEqual(defaultColor, 0)
        for c in range(numColors):
            self.assert_(colorMap.NumElementsWithColor(c+1) in (3,4))

    def testColorMapIndex(self):
        "Test EpetraExt MapColoring-to-ColorMapIndex transform"
        mapColoring   = EpetraExt.CrsGraph_MapColoring(False)
        colorMap      = mapColoring(self.crsg)
        colorMapIndex = EpetraExt.CrsGraph_MapColoringIndex(colorMap)
        columns       = colorMapIndex(self.crsg)
        # No assert test here; just executing has to be enough

####################################################################

if __name__ == "__main__":

    # Create the test suite object
    suite = unittest.TestSuite()

    # Add the test cases to the test suite
    suite.addTest(unittest.makeSuite(ColoringTestCase))

    # Create a communicator
    comm    = Epetra.PyComm()
    iAmRoot = comm.MyPID() == 0

    # Run the test suite
    if iAmRoot: print >>sys.stderr, \
       "\n**************************\nTesting EpetraExt.Coloring\n**************************\n"
    verbosity = options.verbosity * int(iAmRoot)
    result = unittest.TextTestRunner(verbosity=verbosity).run(suite)

    # Exit with a code that indicates the total number of errors and failures
    errsPlusFails = comm.SumAll(len(result.errors) + len(result.failures))
    if errsPlusFails == 0 and iAmRoot: print "End Result: TEST PASSED"
    sys.exit(errsPlusFails)
