#! /usr/bin/env python
# -*- python -*-

# @HEADER
# ************************************************************************
#
#                PyTrilinos: Python Interface to Trilinos
#                   Copyright (2005) Sandia Corporation
#
# Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
# license for use of this work by or on behalf of the U.S. Government.
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# Questions? Contact Bill Spotz (wfspotz@sandia.gov)
#
# ************************************************************************
# @HEADER

#
# System imports
from   optparse import *
import sys
import unittest

#
# Parse the command-line arguments
parser = OptionParser()
parser.add_option("-b", "--use-boost", action="store_true", dest="boost",
                  default=False,
                  help="test the experimental boost-generated PyTrilinos package")
parser.add_option("-t", "--testharness", action="store_true",
                  dest="testharness", default=False,
                  help="test local build modules; prevent loading system-installed modules")
parser.add_option("-v", "--verbosity", type="int", dest="verbosity", default=2,
                  help="set the verbosity level [default 2]")
options,args = parser.parse_args()

#
# Under normal usage, simply use 'from PyTrilinos import Epetra'.  For testing,
# we want to be able to control whether we import from the build directory or
# from a system-installed version of PyTrilinos.
from testutil import fromPyTrilinosImport
Epetra = fromPyTrilinosImport('Epetra', options.testharness)
NOX    = fromPyTrilinosImport('NOX'   , options.testharness)

################################################################################

class StatusTestTestCase(unittest.TestCase):
    "TestCase class for NOX.StatusTest objects"

    def setUp(self):
        self.comm      = Epetra.PyComm()
        self.absResid  = NOX.StatusTest.NormF(1.0e-6)
        self.relResid  = NOX.StatusTest.NormF(1.0e-6,NOX.StatusTest.NormF.Unscaled)
        self.update    = NOX.StatusTest.NormUpdate(1.0e-5)
        self.wrms      = NOX.StatusTest.NormWRMS(1.0e-2, 1.0e-8)
        self.maxIters  = NOX.StatusTest.MaxIters(20)
        self.stagnate  = NOX.StatusTest.Stagnation()
        self.finiteVal = NOX.StatusTest.FiniteValue()

    def tearDown(self):
        self.comm.Barrier()

    def testAbsNormFGetStatus(self):
        "Test NOX.StatusTest.NormF getStatus method"
        self.assertEqual(self.absResid.getStatus(),  NOX.StatusTest.Unevaluated)

    def testRelNormFGetStatus(self):
        "Test NOX.StatusTest.NormF getStatus method"
        self.assertEqual(self.relResid.getStatus(),  NOX.StatusTest.Unevaluated)

    def testNormUpdateGetStatus(self):
        "Test NOX.StatusTest.NormUpdate getStatus method"
        self.assertEqual(self.update.getStatus(),    NOX.StatusTest.Unevaluated)

    def testNormWRMSGetStatus(self):
        "Test NOX.StatusTest.NormWRMS getStatus method"
        self.assertEqual(self.wrms.getStatus(),      NOX.StatusTest.Unconverged)

    def testMaxItersGetStatus(self):
        "Test NOX.StatusTest.MaxIters getStatus method"
        self.assertEqual(self.maxIters.getStatus(),  NOX.StatusTest.Unevaluated)

    def testStagnateGetStatus(self):
        "Test NOX.StatusTest.Stagnate getStatus method"
        self.assertEqual(self.stagnate.getStatus(),  NOX.StatusTest.Unevaluated)

    def testFiniteValueGetStatus(self):
        "Test NOX.StatusTest.FiniteValue getStatus method"
        self.assertEqual(self.finiteVal.getStatus(), NOX.StatusTest.Unevaluated)

    def testComboAnd(self):
        "Test NOX.StatusTest.Combo constructor with AND option"
        converged = NOX.StatusTest.Combo(NOX.StatusTest.Combo.AND)
        converged.addStatusTest(self.absResid)
        converged.addStatusTest(self.relResid)
        converged.addStatusTest(self.wrms    )
        converged.addStatusTest(self.update  )

    def testComboOr(self):
        "Test NOX.StatusTest.Combo constructor with OR option"
        combo = NOX.StatusTest.Combo(NOX.StatusTest.Combo.OR)
        combo.addStatusTest(self.maxIters)
        combo.addStatusTest(self.stagnate)
        combo.addStatusTest(self.finiteVal)

################################################################################

if __name__ == "__main__":

    # Create the test suite object
    suite = unittest.TestSuite()

    # Add the test cases to the test suite
    suite.addTest(unittest.makeSuite(StatusTestTestCase))

    # Create a communicator
    comm    = Epetra.PyComm()
    iAmRoot = comm.MyPID() == 0

    # Run the test suite
    if iAmRoot: print >>sys.stderr, \
       "\n**********************\nTesting NOX.StatusTest\n**********************\n"
    verbosity = options.verbosity * int(iAmRoot)
    result    = unittest.TextTestRunner(verbosity=verbosity).run(suite)

    # Exit with a code that indicates the total number of errors and failures
    errsPlusFails = comm.SumAll(len(result.errors) + len(result.failures))
    if errsPlusFails == 0 and iAmRoot: print "End Result: TEST PASSED"
    sys.exit(errsPlusFails)
