#! /usr/bin/env python
# -*- python -*-

# @HEADER
# ************************************************************************
#
#                PyTrilinos: Python Interface to Trilinos
#                   Copyright (2005) Sandia Corporation
#
# Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
# license for use of this work by or on behalf of the U.S. Government.
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# Questions? Contact Bill Spotz (wfspotz@sandia.gov)
#
# ************************************************************************
# @HEADER

#
# System imports
from   getopt   import getopt
from   numpy    import *
import os
import sys
import unittest

#
# Parse the command-line arguments.  NB: I specifically use the outdated getopt
# here instead of the more modern optparse because I used to get the following
# error:
#
#   python(65587) malloc: *** error for object 0x1715674: Non-aligned pointer being freed
#
# during unit tests that access the Epetra Print() methods and the only way I
# have been able to find to fix the problem has been to use getopt rather than
# optparse.
testharness = False
verbosity   = 2
(options,args) = getopt(sys.argv[1:], "tv:", ["testharness", "verbose="])
for (opt, val) in options:
    if opt in ("-t", "--testharness"): testharness = True
    if opt in ("-v", "--verbosity"  ): verbosity = int(val)
#
# Under normal usage, simply use 'from PyTrilinos import Epetra'.  For testing,
# we want to be able to control whether we import from the build directory or
# from a system-installed version of PyTrilinos.
from testutil import fromPyTrilinosImport
Teuchos = fromPyTrilinosImport('Teuchos', testharness)

##########################################################################

class TeuchosDefaultCommTestCase(unittest.TestCase):
    "TestCase class for DefaultComm communicator objects"

    def setUp(self):
        self.comm  = Teuchos.DefaultComm.getComm()
        if ('MpiComm' in dir(Teuchos)):
            self.commType = "MPI"
            self.output   = "Teuchos::MpiComm<long>{size=%d,rank=%d" % \
                            (self.comm.getSize(), self.comm.getRank())
        else:
            self.commType = "Serial"
            self.output   = "Teuchos::SerialComm<long int>"
        self.comm.barrier()

    def tearDown(self):
        self.comm.barrier()

    def testGetRank(self):
        "Test Teuchos.DefaultComm getRank method"
        pid = self.comm.getRank()
        self.assert_(pid < self.comm.getSize())
        if self.commType == "Serial":
            self.assertEqual(pid, 0)
        else:
            self.assert_(pid >= 0)

    def testGetSize(self):
        "Test Teuchos.DefaultComm getSize method"
        n = self.comm.getSize()
        if self.commType == "Serial":
            self.assertEqual(n, 1)
        else:
            self.assert_(n > 0)

    def testStr(self):
        "Test Teuchos.DefaultComm __str__ method"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        n = len(self.output)
        self.assertEqual(str(self.comm)[:n], self.output)

    def testBroadcastBadArg(self):
        "Test Teuchos.DefaultComm broadcast method with bad argument"
        self.assertRaises(TypeError, self.comm.broadcast, 0, None)

    def testBroadcastNoncontiguous(self):
        "Test Teuchos.DefaultComm broadcast method with noncontiguous array"
        baseArray = zeros((5,5),'d')
        myArray   = baseArray[1:3,1:3]  # Noncontiguous array
        self.assertRaises(TypeError, self.comm.broadcast, 0, myArray)

    def testBroadcastInt(self):
        "Test Teuchos.DefaultComm broadcast method for ints"
        rootVals = array([5,0,4,1,3,2],'i')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],'i')
        self.comm.broadcast(0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastInt2D(self):
        "Test Teuchos.DefaultComm broadcast method for 2D int array"
        rootVals = array([[5,0],[4,1]],'i')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'i')
        self.comm.broadcast(0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastLong(self):
        "Test Teuchos.DefaultComm broadcast method for longs"
        rootVals = array([5,0,4,1,3,2])
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1])
        self.comm.broadcast(0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastLong2D(self):
        "Test Teuchos.DefaultComm broadcast method for 2D long array"
        rootVals = array([[5,0],[4,1]])
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]])
        self.comm.broadcast(0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastFloat(self):
        "Test Teuchos.DefaultComm broadcast method for floats"
        rootVals = array([5,0,4,1,3,2],dtype='f')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],dtype='f')
        self.comm.broadcast(0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastFloat2D(self):
        "Test Teuchos.DefaultComm broadcast method for 2D float array"
        rootVals = array([[5,0],[4,1]],'f')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'f')
        self.comm.broadcast(0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastDouble(self):
        "Test Teuchos.DefaultComm broadcast method for doubles"
        rootVals = array([5,0,4,1,3,2],dtype='d')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],dtype='d')
        self.comm.broadcast(0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastDouble2D(self):
        "Test Teuchos.DefaultComm broadcast method for 2D double array"
        rootVals = array([[5,0],[4,1]],'d')
        if self.comm.getRank() == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'d')
        self.comm.broadcast(0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testGatherAllInt(self):
        "Test Teuchos.DefaultComm gatherAll method for ints"
        myPID      = self.comm.getRank()
        length     = 4*self.comm.getSize()
        globalVals = arange(length, dtype='i')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = self.comm.gatherAll(myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllInt2D(self):
        "Test Teuchos.DefaultComm gatherAll method for 2D int array"
        myPID      = self.comm.getRank()
        numProc    = self.comm.getSize()
        globalVals = zeros((numProc,2,2),'i')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = self.comm.gatherAll(myVals)
        for p in range(numProc):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllIntNoncontiguous(self):
        "Test Teuchos.DefaultComm gatherAll method with noncontiguous int array"
        baseArray = arange(20,dtype='i')
        baseArray.shape = (4,5)
        myArray   = baseArray[1:3,1:4] # Noncontiguous
        allArray  = self.comm.gatherAll(myArray)
        numProc   = self.comm.getSize()
        self.assertEquals(allArray.shape, (numProc,2,3))
        for p in range(numProc):
            self.failUnless((allArray[p,:,:] == myArray).all())

    def testGatherAllLong(self):
        "Test Teuchos.DefaultComm gatherAll method for longs"
        myPID      = self.comm.getRank()
        length     = 4*self.comm.getSize()
        globalVals = arange(length, dtype='i')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = self.comm.gatherAll(myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllLong2D(self):
        "Test Teuchos.DefaultComm gatherAll method for 2D long array"
        myPID      = self.comm.getRank()
        numProc    = self.comm.getSize()
        globalVals = zeros((numProc,2,2),'i')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = self.comm.gatherAll(myVals)
        for p in range(numProc):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllLongNoncontiguous(self):
        "Test Teuchos.DefaultComm gatherAll method with noncontiguous long array"
        baseArray = arange(20)
        baseArray.shape = (4,5)
        myArray   = baseArray[1:3,1:4] # Noncontiguous
        allArray  = self.comm.gatherAll(myArray)
        numProc   = self.comm.getSize()
        self.assertEquals(allArray.shape, (numProc,2,3))
        for p in range(numProc):
            self.failUnless((allArray[p,:,:] == myArray).all())

    def testGatherAllFloat(self):
        "Test Teuchos.DefaultComm gatherAll method for floats"
        myPID      = self.comm.getRank()
        length     = 4*self.comm.getSize()
        globalVals = arange(length, dtype='f')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = self.comm.gatherAll(myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllFloat2D(self):
        "Test Teuchos.DefaultComm gatherAll method for 2D float array"
        myPID      = self.comm.getRank()
        numProc    = self.comm.getSize()
        globalVals = zeros((numProc,2,2),'f')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = self.comm.gatherAll(myVals)
        for p in range(len(globalVals)):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllFloatNoncontiguous(self):
        "Test Teuchos.DefaultComm gatherAll method with noncontiguous float array"
        baseArray = arange(20,dtype='f')
        baseArray.shape = (5,4)
        myArray   = baseArray[1:4,1:3] # Noncontiguous
        allArray  = self.comm.gatherAll(myArray)
        numProc   = self.comm.getSize()
        self.assertEquals(allArray.shape, (numProc,3,2))
        for i in range(numProc):
            self.failUnless((allArray[i,:,:] == myArray).all())

    def testGatherAllDouble(self):
        "Test Teuchos.DefaultComm gatherAll method for doubles"
        myPID      = self.comm.getRank()
        length     = 4*self.comm.getSize()
        globalVals = arange(length, dtype='d')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = self.comm.gatherAll(myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllDouble2D(self):
        "Test Teuchos.DefaultComm gatherAll method for 2D double array"
        myPID      = self.comm.getRank()
        numProc    = self.comm.getSize()
        globalVals = zeros((numProc,2,2),'d')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = self.comm.gatherAll(myVals)
        for p in range(len(globalVals)):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllDoubleNoncontiguous(self):
        "Test Teuchos.DefaultComm gatherAll method with noncontiguous double array"
        baseArray = arange(20,dtype='d')
        baseArray.shape = (5,4)
        myArray   = baseArray[1:4,1:3] # Noncontiguous
        allArray  = self.comm.gatherAll(myArray)
        numProc   = self.comm.getSize()
        self.assertEquals(allArray.shape, (numProc,3,2))
        for i in range(numProc):
            self.failUnless((allArray[i,:,:] == myArray).all())

    def testSumAllInt(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for ints"
        numProc     = self.comm.getSize()
        partialSums = arange(3,dtype='i')
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllInt2D(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for 2D int array"
        numProc     = self.comm.getSize()
        partialSums = arange(12,dtype='i')
        partialSums.shape = (3,4)
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(4):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllIntNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method with noncontiguous int array"
        numProc     = self.comm.getSize()
        baseArray   = arange(15,dtype='i')
        baseArray.shape = (3,5)
        partialSums = baseArray[1:2,1:4]  # Noncontiguous
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(1):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllLong(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for longs"
        numProc     = self.comm.getSize()
        partialSums = arange(3)
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllLong2D(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for 2D long array"
        numProc     = self.comm.getSize()
        partialSums = arange(12)
        partialSums.shape = (3,4)
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(4):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllLongNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method with noncontiguous long array"
        numProc     = self.comm.getSize()
        baseArray   = arange(15)
        baseArray.shape = (3,5)
        partialSums = baseArray[1:2,1:4]  # Noncontiguous
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(1):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllFloat(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for floats"
        numProc     = self.comm.getSize()
        partialSums = arange(4,dtype='f')
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllFloat2D(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for 2D float array"
        numProc     = self.comm.getSize()
        partialSums = arange(15,dtype='f')
        partialSums.shape = (5,3)
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(5):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllFloatNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method with noncontiguous float array"
        numProc     = self.comm.getSize()
        baseArray   = arange(20,dtype='f')
        baseArray.shape = (5,4)
        partialSums = baseArray[1:4,1:3]  # Noncontiguous
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(2):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllDouble(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for doubles"
        numProc     = self.comm.getSize()
        partialSums = arange(4,dtype='d')
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllDouble2D(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method for 2D double array"
        numProc     = self.comm.getSize()
        partialSums = arange(15,dtype='d')
        partialSums.shape = (5,3)
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(5):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllDoubleNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (SUM) method with noncontiguous double array"
        numProc     = self.comm.getSize()
        baseArray   = arange(20,dtype='d')
        baseArray.shape = (5,4)
        partialSums = baseArray[1:4,1:3]  # Noncontiguous
        globalSums  = self.comm.reduceAll(Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(2):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testMaxAllInt(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for ints"
        numProc     = self.comm.getSize()
        baseArray   = arange(5,dtype='i')
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllInt2D(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for 2D int array"
        numProc     = self.comm.getSize()
        baseArray   = arange(9,dtype='i')
        baseArray.shape = (3,3)
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllIntNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method with noncontiguous int array"
        numProc     = self.comm.getSize()
        baseArray   = arange(25,dtype='i')
        baseArray.shape = (5,5)
        baseMaxs    = baseArray[1:4,1:4]  # Noncontiguous
        partialMaxs = baseMaxs * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllLong(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for longs"
        numProc     = self.comm.getSize()
        baseArray   = arange(5)
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllLong2D(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for 2D long array"
        numProc     = self.comm.getSize()
        baseArray   = arange(9)
        baseArray.shape = (3,3)
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllLongNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method with noncontiguous long array"
        numProc     = self.comm.getSize()
        baseArray   = arange(25)
        baseArray.shape = (5,5)
        baseMaxs    = baseArray[1:4,1:4]  # Noncontiguous
        partialMaxs = baseMaxs * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllFloat(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for floats"
        numProc     = self.comm.getSize()
        baseArray   = arange(6,dtype='f')
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllFloat2D(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for 2D float array"
        numProc     = self.comm.getSize()
        baseArray   = arange(16,dtype='f')
        baseArray.shape = (4,4)
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(4):
            for j in range(4):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllFloatNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method with noncontiguous float array"
        numProc     = self.comm.getSize()
        baseArray   = arange(20,dtype='f')
        baseArray.shape = (4,5)
        baseMaxs    = baseArray[1:3,1:4] # Noncontiguous
        partialMaxs = baseMaxs * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(2):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllDouble(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for doubles"
        numProc     = self.comm.getSize()
        baseArray   = arange(6,dtype='d')
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllDouble2D(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method for 2D double array"
        numProc     = self.comm.getSize()
        baseArray   = arange(16,dtype='d')
        baseArray.shape = (4,4)
        partialMaxs = array(baseArray) * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(4):
            for j in range(4):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllDoubleNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MAX) method with noncontiguous double array"
        numProc     = self.comm.getSize()
        baseArray   = arange(20,dtype='d')
        baseArray.shape = (4,5)
        baseMaxs    = baseArray[1:3,1:4] # Noncontiguous
        partialMaxs = baseMaxs * (self.comm.getRank()+1)
        globalMaxs  = self.comm.reduceAll(Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(2):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMinAllInt(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for ints"
        baseArray   = arange(5,dtype='i')
        partialMins = array(baseArray) * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllInt2D(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for 2D int array"
        baseArray    = arange(18,dtype="i")
        baseArray.shape = (3,6)
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(6):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllIntNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method with noncontiguous int array"
        baseArray   = arange(24,dtype='i')
        baseArray.shape = (4,6)
        baseArray    = baseArray[1:3,1:5]  # Noncontiguous
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(2):
            for j in range(4):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllLong(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for longs"
        baseArray   = arange(5)
        partialMins = array(baseArray) * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllLong2D(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for 2D long array"
        baseArray    = arange(18)
        baseArray.shape = (3,6)
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(6):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllLongNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method with noncontiguous long array"
        baseArray   = arange(24)
        baseArray.shape = (4,6)
        baseArray    = baseArray[1:3,1:5]  # Noncontiguous
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(2):
            for j in range(4):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllFloat(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for floats"
        baseArray   = arange(4,dtype='f')
        partialMins = array(baseArray) * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllFloat2D(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for 2D float array"
        baseArray    = arange(30,dtype='f')
        baseArray.shape = (6,5)
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(6):
            for j in range(5):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllFloatNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method with noncontiguous float array"
        baseArray   = arange(36,dtype='f')
        baseArray.shape = (6,6)
        baseArray    = baseArray[1:4,1:4]  # Noncontiguous
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllDouble(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for doubles"
        baseArray   = arange(4,dtype='d')
        partialMins = array(baseArray) * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllDouble2D(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method for 2D double array"
        baseArray    = arange(30,dtype='d')
        baseArray.shape = (6,5)
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(6):
            for j in range(5):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllDoubleNoncontiguous(self):
        "Test Teuchos.DefaultComm reduceAll (MIN) method with noncontiguous double array"
        baseArray   = arange(36,dtype='d')
        baseArray.shape = (6,6)
        baseArray    = baseArray[1:4,1:4]  # Noncontiguous
        partialMins = baseArray * (self.comm.getRank()+1)
        globalMins  = self.comm.reduceAll(Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testScanSumInt(self):
        "Test Teuchos.DefaultComm scan (SUM) method for ints"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(3,dtype='i')
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumInt2D(self):
        "Test Teuchos.DefaultComm scan (SUM) method for 2D int array"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(12,dtype='i')
        myVals.shape = (3,4)
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(4):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumIntNoncontiguous(self):
        "Test Teuchos.DefaultComm scan (SUM) method with noncontiguous int array"
        myPIN    = self.comm.getRank() + 1
        baseArray = arange(15,dtype='i')
        baseArray.shape = (3,5)
        myVals    = baseArray[1:2,1:4]  # Noncontiguous
        scanSums  = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(1):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumLong(self):
        "Test Teuchos.DefaultComm scan (SUM) method for longs"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(3)
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumLong2D(self):
        "Test Teuchos.DefaultComm scan (SUM) method for 2D long array"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(12)
        myVals.shape = (3,4)
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(4):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumLongNoncontiguous(self):
        "Test Teuchos.DefaultComm scan (SUM) method with noncontiguous long array"
        myPIN    = self.comm.getRank() + 1
        baseArray = arange(15)
        baseArray.shape = (3,5)
        myVals    = baseArray[1:2,1:4]  # Noncontiguous
        scanSums  = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(1):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumFloat(self):
        "Test Teuchos.DefaultComm scan (SUM) method for floats"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(4,dtype='f')
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumFloat2D(self):
        "Test Teuchos.DefaultComm scan (SUM) method for 2D float array"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(15,dtype='f')
        myVals.shape = (5,3)
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(5):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumFloatNoncontiguous(self):
        "Test Teuchos.DefaultComm scan (SUM) method with noncontiguous float array"
        myPIN    = self.comm.getRank() + 1
        baseArray = arange(20,dtype='f')
        baseArray.shape = (5,4)
        myVals    = baseArray[1:4,1:3]  # Noncontiguous
        scanSums  = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(2):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumDouble(self):
        "Test Teuchos.DefaultComm scan (SUM) method for doubles"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(4,dtype='d')
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumDouble2D(self):
        "Test Teuchos.DefaultComm scan (SUM) method for 2D double array"
        myPIN    = self.comm.getRank() + 1
        myVals   = arange(15,dtype='d')
        myVals.shape = (5,3)
        scanSums = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(5):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumDoubleNoncontiguous(self):
        "Test Teuchos.DefaultComm scan (SUM) method with noncontiguous double array"
        myPIN    = self.comm.getRank() + 1
        baseArray = arange(20,dtype='d')
        baseArray.shape = (5,4)
        myVals    = baseArray[1:4,1:3]  # Noncontiguous
        scanSums  = self.comm.scan(Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(2):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

##########################################################################

class TeuchosDefaultCommHelpersTestCase(unittest.TestCase):
    "TestCase class for DefaultComm communicator helpers"

    def setUp(self):
        self.comm  = Teuchos.DefaultComm.getComm()
        if self.comm.getSize() == 1:
            self.commType = "Serial"
        else:
            self.commType = "MPI"
        Teuchos.barrier(self.comm)

    def tearDown(self):
        Teuchos.barrier(self.comm)

    def testRank(self):
        "Test Teuchos rank helper function"
        pid = Teuchos.rank(self.comm)
        self.assert_(pid < Teuchos.size(self.comm))
        if self.commType == "Serial":
            self.assertEqual(pid, 0)
        else:
            self.assert_(pid >= 0)

    def testSize(self):
        "Test Teuchos size helper function"
        n = Teuchos.size(self.comm)
        if self.commType == "Serial":
            self.assertEqual(n, 1)
        else:
            self.assert_(n > 0)

    def testBroadcastBadArg(self):
        "Test Teuchos broadcast helper function with bad argument"
        self.assertRaises(TypeError, Teuchos.broadcast, self.comm, 0, None)

    def testBroadcastNoncontiguous(self):
        "Test Teuchos broadcast helper function with noncontiguous array"
        baseArray = zeros((5,5),'d')
        myArray   = baseArray[1:3,1:3]  # Noncontiguous array
        self.assertRaises(TypeError, Teuchos.broadcast, self.comm, 0, myArray)

    def testBroadcastInt(self):
        "Test Teuchos broadcast helper function for ints"
        rootVals = array([5,0,4,1,3,2],'i')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],'i')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastInt2D(self):
        "Test Teuchos broadcast helper function for 2D int array"
        rootVals = array([[5,0],[4,1]],'i')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'i')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastLong(self):
        "Test Teuchos broadcast helper function for longs"
        rootVals = array([5,0,4,1,3,2])
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1])
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastLong2D(self):
        "Test Teuchos broadcast helper function for 2D long array"
        rootVals = array([[5,0],[4,1]])
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]])
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastFloat(self):
        "Test Teuchos broadcast helper function for floats"
        rootVals = array([5,0,4,1,3,2],dtype='f')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],dtype='f')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastFloat2D(self):
        "Test Teuchos broadcast helper function for 2D float array"
        rootVals = array([[5,0],[4,1]],'f')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'f')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testBroadcastDouble(self):
        "Test Teuchos broadcast helper function for doubles"
        rootVals = array([5,0,4,1,3,2],dtype='d')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([-1,-1,-1,-1,-1,-1],dtype='d')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(len(rootVals)):
            self.assertEquals(myVals[i], rootVals[i])

    def testBroadcastDouble2D(self):
        "Test Teuchos broadcast helper function for 2D double array"
        rootVals = array([[5,0],[4,1]],'d')
        if Teuchos.rank(self.comm) == 0:
            myVals = array(rootVals)
        else:
            myVals = array([[-1,-1],[-1,-1]],'d')
        Teuchos.broadcast(self.comm, 0, myVals)
        for i in range(2):
            for j in range(2):
                self.assertEquals(myVals[i,j], rootVals[i,j])

    def testGatherAllInt(self):
        "Test Teuchos gatherAll helper function for ints"
        myPID      = Teuchos.rank(self.comm)
        length     = 4*Teuchos.size(self.comm)
        globalVals = arange(length, dtype='i')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllInt2D(self):
        "Test Teuchos gatherAll helper function for 2D int array"
        myPID      = Teuchos.rank(self.comm)
        numProc    = Teuchos.size(self.comm)
        globalVals = zeros((numProc,2,2),'i')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        for p in range(numProc):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllIntNoncontiguous(self):
        "Test Teuchos gatherAll helper function with noncontiguous int array"
        baseArray = arange(20,dtype='i')
        baseArray.shape = (4,5)
        myArray   = baseArray[1:3,1:4] # Noncontiguous
        allArray  = Teuchos.gatherAll(self.comm, myArray)
        numProc   = Teuchos.size(self.comm)
        self.assertEquals(allArray.shape, (numProc,2,3))
        for p in range(numProc):
            self.failUnless((allArray[p,:,:] == myArray).all())

    def testGatherAllLong(self):
        "Test Teuchos gatherAll helper function for longs"
        myPID      = Teuchos.rank(self.comm)
        length     = 4*Teuchos.size(self.comm)
        globalVals = arange(length, dtype='i')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllLong2D(self):
        "Test Teuchos gatherAll helper function for 2D long array"
        myPID      = Teuchos.rank(self.comm)
        numProc    = Teuchos.size(self.comm)
        globalVals = zeros((numProc,2,2),'i')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        for p in range(numProc):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllLongNoncontiguous(self):
        "Test Teuchos gatherAll helper function with noncontiguous long array"
        baseArray = arange(20)
        baseArray.shape = (4,5)
        myArray   = baseArray[1:3,1:4] # Noncontiguous
        allArray  = Teuchos.gatherAll(self.comm, myArray)
        numProc   = Teuchos.size(self.comm)
        self.assertEquals(allArray.shape, (numProc,2,3))
        for p in range(numProc):
            self.failUnless((allArray[p,:,:] == myArray).all())

    def testGatherAllFloat(self):
        "Test Teuchos gatherAll helper function for floats"
        myPID      = Teuchos.rank(self.comm)
        length     = 4*Teuchos.size(self.comm)
        globalVals = arange(length, dtype='f')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllFloat2D(self):
        "Test Teuchos gatherAll helper function for 2D float array"
        myPID      = Teuchos.rank(self.comm)
        numProc    = Teuchos.size(self.comm)
        globalVals = zeros((numProc,2,2),'f')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        for p in range(len(globalVals)):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllFloatNoncontiguous(self):
        "Test Teuchos gatherAll helper function with noncontiguous float array"
        baseArray = arange(20,dtype='f')
        baseArray.shape = (5,4)
        myArray   = baseArray[1:4,1:3] # Noncontiguous
        allArray  = Teuchos.gatherAll(self.comm, myArray)
        numProc   = Teuchos.size(self.comm)
        self.assertEquals(allArray.shape, (numProc,3,2))
        for i in range(numProc):
            self.failUnless((allArray[i,:,:] == myArray).all())

    def testGatherAllDouble(self):
        "Test Teuchos gatherAll helper function for doubles"
        myPID      = Teuchos.rank(self.comm)
        length     = 4*Teuchos.size(self.comm)
        globalVals = arange(length, dtype='d')
        myVals     = array(globalVals[4*myPID:4*(myPID+1)])
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        allVals.shape = (length,)
        for i in range(len(globalVals)):
            self.assertEquals(allVals[i], globalVals[i])

    def testGatherAllDouble2D(self):
        "Test Teuchos gatherAll helper function for 2D double array"
        myPID      = Teuchos.rank(self.comm)
        numProc    = Teuchos.size(self.comm)
        globalVals = zeros((numProc,2,2),'d')
        for i in range(numProc):
            globalVals[i,:,:] = [[1,2],[3,4]]
        myVals     = globalVals[myPID,:,:]
        allVals    = Teuchos.gatherAll(self.comm, myVals)
        for p in range(len(globalVals)):
            for i in range(2):
                for j in range(2):
                    self.assertEquals(allVals[p,i,j], globalVals[p,i,j])

    def testGatherAllDoubleNoncontiguous(self):
        "Test Teuchos gatherAll helper function with noncontiguous double array"
        baseArray = arange(20,dtype='d')
        baseArray.shape = (5,4)
        myArray   = baseArray[1:4,1:3] # Noncontiguous
        allArray  = Teuchos.gatherAll(self.comm, myArray)
        numProc   = Teuchos.size(self.comm)
        self.assertEquals(allArray.shape, (numProc,3,2))
        for i in range(numProc):
            self.failUnless((allArray[i,:,:] == myArray).all())

    def testSumAllInt(self):
        "Test Teuchos reduceAll (SUM) helper function for ints"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(3,dtype='i')
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllInt2D(self):
        "Test Teuchos reduceAll (SUM) helper function for 2D int array"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(12,dtype='i')
        partialSums.shape = (3,4)
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(4):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllIntNoncontiguous(self):
        "Test Teuchos reduceAll (SUM) helper function with noncontiguous int array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(15,dtype='i')
        baseArray.shape = (3,5)
        partialSums = baseArray[1:2,1:4]  # Noncontiguous
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(1):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllLong(self):
        "Test Teuchos reduceAll (SUM) helper function for longs"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(3)
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllLong2D(self):
        "Test Teuchos reduceAll (SUM) helper function for 2D long array"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(12)
        partialSums.shape = (3,4)
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(4):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllLongNoncontiguous(self):
        "Test Teuchos reduceAll (SUM) helper function with noncontiguous long array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(15)
        baseArray.shape = (3,5)
        partialSums = baseArray[1:2,1:4]  # Noncontiguous
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(1):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllFloat(self):
        "Test Teuchos reduceAll (SUM) helper function for floats"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(4,dtype='f')
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllFloat2D(self):
        "Test Teuchos reduceAll (SUM) helper function for 2D float array"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(15,dtype='f')
        partialSums.shape = (5,3)
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(5):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllFloatNoncontiguous(self):
        "Test Teuchos reduceAll (SUM) helper function with noncontiguous float array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(20,dtype='f')
        baseArray.shape = (5,4)
        partialSums = baseArray[1:4,1:3]  # Noncontiguous
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(2):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllDouble(self):
        "Test Teuchos reduceAll (SUM) helper function for doubles"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(4,dtype='d')
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(len(partialSums)):
            self.assertEquals(globalSums[i], numProc*partialSums[i])

    def testSumAllDouble2D(self):
        "Test Teuchos reduceAll (SUM) helper function for 2D double array"
        numProc     = Teuchos.size(self.comm)
        partialSums = arange(15,dtype='d')
        partialSums.shape = (5,3)
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(5):
            for j in range(3):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testSumAllDoubleNoncontiguous(self):
        "Test Teuchos reduceAll (SUM) helper function with noncontiguous double array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(20,dtype='d')
        baseArray.shape = (5,4)
        partialSums = baseArray[1:4,1:3]  # Noncontiguous
        globalSums  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_SUM, partialSums)
        for i in range(3):
            for j in range(2):
                self.assertEquals(globalSums[i,j], numProc*partialSums[i,j])

    def testMaxAllInt(self):
        "Test Teuchos reduceAll (MAX) helper function for ints"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(5,dtype='i')
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllInt2D(self):
        "Test Teuchos reduceAll (MAX) helper function for 2D int array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(9,dtype='i')
        baseArray.shape = (3,3)
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllIntNoncontiguous(self):
        "Test Teuchos reduceAll (MAX) helper function with noncontiguous int array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(25,dtype='i')
        baseArray.shape = (5,5)
        baseMaxs    = baseArray[1:4,1:4]  # Noncontiguous
        partialMaxs = baseMaxs * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllLong(self):
        "Test Teuchos reduceAll (MAX) helper function for longs"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(5)
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllLong2D(self):
        "Test Teuchos reduceAll (MAX) helper function for 2D long array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(9)
        baseArray.shape = (3,3)
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllLongNoncontiguous(self):
        "Test Teuchos reduceAll (MAX) helper function with noncontiguous long array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(25)
        baseArray.shape = (5,5)
        baseMaxs    = baseArray[1:4,1:4]  # Noncontiguous
        partialMaxs = baseMaxs * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllFloat(self):
        "Test Teuchos reduceAll (MAX) helper function for floats"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(6,dtype='f')
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllFloat2D(self):
        "Test Teuchos reduceAll (MAX) helper function for 2D float array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(16,dtype='f')
        baseArray.shape = (4,4)
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(4):
            for j in range(4):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllFloatNoncontiguous(self):
        "Test Teuchos reduceAll (MAX) helper function with noncontiguous float array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(20,dtype='f')
        baseArray.shape = (4,5)
        baseMaxs    = baseArray[1:3,1:4] # Noncontiguous
        partialMaxs = baseMaxs * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(2):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMaxAllDouble(self):
        "Test Teuchos reduceAll (MAX) helper function for doubles"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(6,dtype='d')
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(len(baseArray)):
            self.assertEquals(globalMaxs[i], numProc*baseArray[i])

    def testMaxAllDouble2D(self):
        "Test Teuchos reduceAll (MAX) helper function for 2D double array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(16,dtype='d')
        baseArray.shape = (4,4)
        partialMaxs = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(4):
            for j in range(4):
                self.assertEquals(globalMaxs[i,j], numProc*baseArray[i,j])

    def testMaxAllDoubleNoncontiguous(self):
        "Test Teuchos reduceAll (MAX) helper function with noncontiguous double array"
        numProc     = Teuchos.size(self.comm)
        baseArray   = arange(20,dtype='d')
        baseArray.shape = (4,5)
        baseMaxs    = baseArray[1:3,1:4] # Noncontiguous
        partialMaxs = baseMaxs * (Teuchos.rank(self.comm)+1)
        globalMaxs  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MAX, partialMaxs)
        for i in range(2):
            for j in range(3):
                self.assertEquals(globalMaxs[i,j], numProc*baseMaxs[i,j])

    def testMinAllInt(self):
        "Test Teuchos reduceAll (MIN) helper function for ints"
        baseArray   = arange(5,dtype='i')
        partialMins = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllInt2D(self):
        "Test Teuchos reduceAll (MIN) helper function for 2D int array"
        baseArray    = arange(18,dtype="i")
        baseArray.shape = (3,6)
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(6):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllIntNoncontiguous(self):
        "Test Teuchos reduceAll (MIN) helper function with noncontiguous int array"
        baseArray   = arange(24,dtype='i')
        baseArray.shape = (4,6)
        baseArray    = baseArray[1:3,1:5]  # Noncontiguous
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(2):
            for j in range(4):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllLong(self):
        "Test Teuchos reduceAll (MIN) helper function for longs"
        baseArray   = arange(5)
        partialMins = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllLong2D(self):
        "Test Teuchos reduceAll (MIN) helper function for 2D long array"
        baseArray    = arange(18)
        baseArray.shape = (3,6)
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(6):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllLongNoncontiguous(self):
        "Test Teuchos reduceAll (MIN) helper function with noncontiguous long array"
        baseArray   = arange(24)
        baseArray.shape = (4,6)
        baseArray    = baseArray[1:3,1:5]  # Noncontiguous
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(2):
            for j in range(4):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllFloat(self):
        "Test Teuchos reduceAll (MIN) helper function for floats"
        baseArray   = arange(4,dtype='f')
        partialMins = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllFloat2D(self):
        "Test Teuchos reduceAll (MIN) helper function for 2D float array"
        baseArray    = arange(30,dtype='f')
        baseArray.shape = (6,5)
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(6):
            for j in range(5):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllFloatNoncontiguous(self):
        "Test Teuchos reduceAll (MIN) helper function with noncontiguous float array"
        baseArray   = arange(36,dtype='f')
        baseArray.shape = (6,6)
        baseArray    = baseArray[1:4,1:4]  # Noncontiguous
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllDouble(self):
        "Test Teuchos reduceAll (MIN) helper function for doubles"
        baseArray   = arange(4,dtype='d')
        partialMins = array(baseArray) * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(len(baseArray)):
            self.assertEquals(globalMins[i], baseArray[i])

    def testMinAllDouble2D(self):
        "Test Teuchos reduceAll (MIN) helper function for 2D double array"
        baseArray    = arange(30,dtype='d')
        baseArray.shape = (6,5)
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(6):
            for j in range(5):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testMinAllDoubleNoncontiguous(self):
        "Test Teuchos reduceAll (MIN) helper function with noncontiguous double array"
        baseArray   = arange(36,dtype='d')
        baseArray.shape = (6,6)
        baseArray    = baseArray[1:4,1:4]  # Noncontiguous
        partialMins = baseArray * (Teuchos.rank(self.comm)+1)
        globalMins  = Teuchos.reduceAll(self.comm, Teuchos.REDUCE_MIN, partialMins)
        for i in range(3):
            for j in range(3):
                self.assertEquals(globalMins[i,j], baseArray[i,j])

    def testScanSumInt(self):
        "Test Teuchos scan (SUM) helper function for ints"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(3,dtype='i')
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumInt2D(self):
        "Test Teuchos scan (SUM) helper function for 2D int array"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(12,dtype='i')
        myVals.shape = (3,4)
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(4):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumIntNoncontiguous(self):
        "Test Teuchos scan (SUM) helper function with noncontiguous int array"
        myPIN    = Teuchos.rank(self.comm) + 1
        baseArray = arange(15,dtype='i')
        baseArray.shape = (3,5)
        myVals    = baseArray[1:2,1:4]  # Noncontiguous
        scanSums  = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(1):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumLong(self):
        "Test Teuchos scan (SUM) helper function for longs"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(3)
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumLong2D(self):
        "Test Teuchos scan (SUM) helper function for 2D long array"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(12)
        myVals.shape = (3,4)
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(4):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumLongNoncontiguous(self):
        "Test Teuchos scan (SUM) helper function with noncontiguous long array"
        myPIN    = Teuchos.rank(self.comm) + 1
        baseArray = arange(15)
        baseArray.shape = (3,5)
        myVals    = baseArray[1:2,1:4]  # Noncontiguous
        scanSums  = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(1):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumFloat(self):
        "Test Teuchos scan (SUM) helper function for floats"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(4,dtype='f')
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumFloat2D(self):
        "Test Teuchos scan (SUM) helper function for 2D float array"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(15,dtype='f')
        myVals.shape = (5,3)
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(5):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumFloatNoncontiguous(self):
        "Test Teuchos scan (SUM) helper function with noncontiguous float array"
        myPIN    = Teuchos.rank(self.comm) + 1
        baseArray = arange(20,dtype='f')
        baseArray.shape = (5,4)
        myVals    = baseArray[1:4,1:3]  # Noncontiguous
        scanSums  = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(2):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumDouble(self):
        "Test Teuchos scan (SUM) helper function for doubles"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(4,dtype='d')
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(len(myVals)):
            self.assertEquals(scanSums[i], myPIN*myVals[i])

    def testScanSumDouble2D(self):
        "Test Teuchos scan (SUM) helper function for 2D double array"
        myPIN    = Teuchos.rank(self.comm) + 1
        myVals   = arange(15,dtype='d')
        myVals.shape = (5,3)
        scanSums = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(5):
            for j in range(3):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

    def testScanSumDoubleNoncontiguous(self):
        "Test Teuchos scan (SUM) helper function with noncontiguous double array"
        myPIN    = Teuchos.rank(self.comm) + 1
        baseArray = arange(20,dtype='d')
        baseArray.shape = (5,4)
        myVals    = baseArray[1:4,1:3]  # Noncontiguous
        scanSums  = Teuchos.scan(self.comm, Teuchos.REDUCE_SUM, myVals)
        for i in range(3):
            for j in range(2):
                self.assertEquals(scanSums[i,j], myPIN*myVals[i,j])

##########################################################################

if __name__ == "__main__":

    # Create the test suite object
    suite = unittest.TestSuite()

    # Add the test cases to the test suite
    suite.addTest(unittest.makeSuite(TeuchosDefaultCommTestCase       ))
    suite.addTest(unittest.makeSuite(TeuchosDefaultCommHelpersTestCase))

    # Create a communicator
    comm    = Teuchos.DefaultComm.getComm()
    iAmRoot = comm.getRank() == 0

    # Run the test suite
    if iAmRoot: print >>sys.stderr, \
          "\n********************\nTesting Teuchos.Comm\n********************\n"
    v = verbosity * int(iAmRoot)
    result = unittest.TextTestRunner(verbosity=v).run(suite)

    # Exit with a code that indicates the total number of errors and failures
    errsPlusFails = comm.reduceAll(Teuchos.REDUCE_SUM,
                                   len(result.errors) + len(result.failures))
    if errsPlusFails == 0 and iAmRoot: print >>sys.stderr, "End Result: TEST PASSED"
    sys.exit(errsPlusFails)
