#! /usr/bin/env python
# -*- python -*-

# @HEADER
# ************************************************************************
#
#                PyTrilinos: Python Interface to Trilinos
#                   Copyright (2005) Sandia Corporation
#
# Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
# license for use of this work by or on behalf of the U.S. Government.
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# Questions? Contact Bill Spotz (wfspotz@sandia.gov)
#
# ************************************************************************
# @HEADER

#
# System imports
from   getopt   import getopt
import sys
import unittest

#
# Parse the command-line arguments.  NB: I specifically use the outdated getopt
# here instead of the more modern optparse because I used to get the following
# error:
#
#   python(65587) malloc: *** error for object 0x1715674: Non-aligned pointer being freed
#
# during unit tests that access the Epetra Print() methods and the only way I
# have been able to find to fix the problem has been to use getopt rather than
# optparse.
testharness = False
verbosity   = 2
(options,args) = getopt(sys.argv[1:], "tv:", ["testharness", "verbose="])
for (opt, val) in options:
    if opt in ("-t", "--testharness"): testharness = True
    if opt in ("-v", "--verbosity"  ): verbosity = int(val)

#
# Under normal usage, simply use 'from PyTrilinos import Teuchos'.  For testing,
# we want to be able to control whether we import from the build directory or
# from a system-installed version of PyTrilinos.
from testutil import fromPyTrilinosImport
Teuchos = fromPyTrilinosImport('Teuchos', testharness)

####################################################################

class ParameterListTestCase(unittest.TestCase):
    "TestCase class for Teuchos.ParameterList"

    def setUp(self):
        self.comm  = Teuchos.DefaultComm.getComm()
        if ('MpiComm' in dir(Teuchos)):
            self.commType = "MPI"
        else:
            self.commType = "Serial"
        self.plist = Teuchos.ParameterList()
        self.name  = "Solver Params"
        self.dict  = {"b" : False,
                      "i" : 2,
                      "f" : 3.14,
                      "d" : {"a":1, "b":2},
                      "s" : "Sandia"
                      }
        self.fName = "testParameterList%d.dat" % self.comm.getRank()
        self.comm.barrier()

    def tearDown(self):
        self.comm.barrier()

    def testConstructor0(self):
        "Test Teuchos.ParameterList default constructor"
        self.assertEqual(isinstance(self.plist,Teuchos.ParameterList), True)

    def testConstructor1(self):
        "Test Teuchos.ParameterList string constructor"
        plist = Teuchos.ParameterList(self.name)
        self.assertEqual(isinstance(plist,Teuchos.ParameterList), True)
        self.assertEqual(plist.name(), self.name)

    def testConstructor2(self):
        "Test Teuchos.ParameterList dictionary constructor"
        d = {"i" : 5,
             "f" : 3.14,
             "s" : "New Mexico"
             }
        plist = Teuchos.ParameterList(d)
        for key in d.keys():
            self.assertEqual(plist.get(key), d[key])

    def testConstructor3(self):
        "Test Teuchos.ParameterList nested dictionary constructor"
        plist = Teuchos.ParameterList(self.dict)
        for key in self.dict.keys():
            self.assertEqual(plist.get(key), self.dict[key])

    def testConstructor4(self):
        "Test Teuchos.ParameterList dictionary constructor, bad key"
        d = {"i" : 5,
             "f" : 3.14,
             "s" : "New Mexico",
             1   : "Should fail"
             }
        self.assertRaises(TypeError, Teuchos.ParameterList, d)

    def testConstructor5(self):
        "Test Teuchos.ParameterList dictionary constructor, bad value"
        d = {"i" : 5,
             "f" : 3.14,
             "s" : "New Mexico",
             "b" : (1,2,3)
             }
        self.assertRaises(ValueError, Teuchos.ParameterList, d)

    def testConstructor6(self):
        "Test Teuchos.ParameterList copy constructor"
        plist_copy = Teuchos.ParameterList(self.plist)
        self.assertEqual(isinstance(plist_copy,Teuchos.ParameterList), True)

    def testConstructor7(self):
        "Test Teuchos.ParameterList constructor with bad argument"
        self.assertRaises(ValueError, Teuchos.ParameterList, 2.718)

    def testSetName(self):
        "Test Teuchos.ParameterList name and setName methods"
        self.assertEqual(self.plist.name(), "ANONYMOUS")
        self.plist.setName(self.name)
        self.assertEqual(self.plist.name(), self.name)

    def testSetParameters(self):
        "Test Teuchos.ParameterList setParameters method"
        intName    = "int parameter"
        intValue   = 8
        floatName  = "float parameter"
        floatValue = 3.14
        self.plist.set(intName,  intValue  )
        self.plist.set(floatName,floatValue)
        newList = Teuchos.ParameterList()
        newList.setParameters(self.plist)
        self.assertEqual(newList.get(intName  ), intValue  )
        self.assertEqual(newList.get(floatName), floatValue)

    def testSetInt(self):
        "Test Teuchos.ParameterList set and get methods for an integer"
        name  = "int parameter"
        value = 12
        self.plist.set(name, value)
        self.assertEqual(self.plist.get(name), value)

    def testSetFloat(self):
        "Test Teuchos.ParameterList set and get methods for a float"
        name  = "float parameter"
        value = 12.0
        self.plist.set(name, value)
        self.assertEqual(self.plist.get(name), value)

    def testSetString(self):
        "Test Teuchos.ParameterList set and get methods for a string"
        name  = "string parameter"
        value = "12"
        self.plist.set(name, value)
        self.assertEqual(self.plist.get(name), value)

    def testSetDictionary(self):
        "Test Teuchos.ParameterList set method for a dictionary"
        name = "dict parameter"
        self.plist.set(name, self.dict)
        sublist = self.plist.get(name)
        self.assertEqual(isinstance(sublist, Teuchos.ParameterList), True)
        for key in self.dict.keys():
            self.assertEqual(sublist.get(key), self.dict[key])
 
    def testSetParameterList(self):
        "Test Teuchos.ParameterList set and get methods for a ParameterList"
        name    = "sublist"
        sublist = Teuchos.ParameterList()
        self.plist.set(name, sublist)
        self.assertEqual(isinstance(self.plist.get(name),
                                    Teuchos.ParameterList), True)
        self.assertEqual(self.plist.isSublist(name), True)

    def testSetNone(self):
        "Test Teuchos.ParameterList set method for None"
        self.assertRaises(TypeError, self.plist.set, "bad parameter", None)

    def testSetBadType(self):
        "Test Teuchos.ParameterList set method for an unsupported type"
        self.assertRaises(TypeError, self.plist.set, "bad parameter", (1,2,3))

    def testSetBadKey(self):
        "Test Teuchos.ParameterList set method with bad key"
        self.assertRaises(TypeError, self.plist.set, 1, "one")

    def testSetParameters1(self):
        "Test Teuchos.ParameterList setParameters method for a dictionary"
        self.plist.setParameters(self.dict)
        self.assertEqual(self.plist, self.dict)

    def testSetParameters2(self):
        "Test Teuchos.ParameterList setParameters method for a ParameterList"
        plist = Teuchos.ParameterList()
        plist.set("int parameter"  , 8)
        plist.set("float parameter", 3.14)
        self.plist.setParameters(plist)
        self.assertEqual(self.plist, plist)

    def testSetParametersBad(self):
        "Test Teuchos.ParameterList setParameters method for unsupported type"
        d = dict(i=5,f=10.0,t=(1,2,3,5,7))
        self.assertRaises(ValueError, self.plist.setParameters, d)

    def testGetDefault(self):
        "Test Teuchos.ParameterList get method using default value"
        default = None
        self.assertEqual(self.plist.get("junk",default), default)

    def testSublistNew(self):
        "Test Teuchos.ParameterList sublist method for new sublist"
        sublist = self.plist.sublist("new")
        self.assertEqual(isinstance(sublist, Teuchos.ParameterList), True)
        sublist = self.plist.get("new")
        self.assertEqual(isinstance(sublist, Teuchos.ParameterList), True)

    def testSublistOld(self):
        "Test Teuchos.ParameterList sublist method for existing sublist"
        self.plist.set("old",self.dict)
        sublist = self.plist.sublist("old")
        self.assertEqual(isinstance(sublist, Teuchos.ParameterList), True)

    def testSublistBad(self):
        "Test Teuchos.ParameterList sublist method for non-sublist"
        self.plist.set("new", 1)
        self.assertRaises(TypeError, self.plist.sublist, "new")

    def testIsParameterTrue(self):
        "Test Teuchos.ParameterList isParameter method existing parameter"
        name = "string parameter"
        self.plist.set(name,"Hello")
        self.assertEqual(self.plist.isParameter(name), True)

    def testIsParameterFalse(self):
        "Test Teuchos.ParameterList isParameter method nonexisting parameter"
        name = "parameter"
        self.assertEqual(self.plist.isParameter(name), False)

    def testIsSublistTrue(self):
        "Test Teuchos.ParameterList isSublist method for existing sublist"
        name = "sublist"
        self.plist.sublist(name)
        self.assertEqual(self.plist.isSublist(name), True)

    def testIsSublistFalse1(self):
        "Test Teuchos.ParameterList isSublist method for existing non-sublist parameter"
        name = "string parameter"
        self.plist.set(name,"Hello")
        self.assertEqual(self.plist.isSublist(name), False)

    def testIsSublistFalse2(self):
        "Test Teuchos.ParameterList isSublist method for nonexisting parameter"
        name = "parameter"
        self.assertEqual(self.plist.isSublist(name), False)

    def testPrint0(self):
        "Test Teuchos.ParameterList _print method for empty list"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        f = open(self.fName, "w")
        self.plist._print(f)
        f.close()
        self.assertEqual(open(self.fName,"r").read(), "[empty list]\n")

    def testPrint1(self):
        "Test Teuchos.ParameterList _print method for non-empty list"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        names  = ["max its","tolerance"]
        values = [100      , 1e-6      ]
        for i in range(len(names)):
            self.plist.set(names[i], values[i])
        f = open(self.fName, "w")
        self.plist._print(f)
        f.close()
        lines = open(self.fName,"r").readlines()
        for i in range(len(lines)):
            self.assertEqual(lines[i], "%s = %g   [unused]\n" % (names[i], values[i]))

    def testPrint2(self):
        "Test Teuchos.ParameterList _print method for non-empty list and indentation"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        names  = ["max its","tolerance"]
        values = [100      , 1e-6      ]
        for i in range(len(names)):
            self.plist.set(names[i], values[i])
        f = open(self.fName, "w")
        self.plist._print(f,2)
        f.close()
        lines = open(self.fName,"r").readlines()
        for i in range(len(lines)):
            self.assertEqual(lines[i].strip(), "%s = %g   [unused]" %
                             (names[i], values[i]))

    def testPrint3(self):
        "Test Teuchos.ParameterList _print method for non-empty list, indentation and types"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        names  = ["max its","tolerance"]
        values = [100      , 1e-6      ]
        types  = ["int"    ,"double"   ]
        for i in range(len(names)):
            self.plist.set(names[i], values[i])
        f = open(self.fName, "w")
        self.plist._print(f,2,True)
        f.close()
        lines = open(self.fName,"r").readlines()
        for i in range(len(lines)):
            self.assertEqual(lines[i].strip(), "%s : %s = %g   [unused]" %
                             (names[i], types[i], values[i]))

    def testUnused(self):
        "Test Teuchos.ParameterList unused method"
        names  = ["s1"   , "s2"   , "s3"         ]
        values = ["Hello", "World", "Albuquerque"]
        for i in range(len(names)):
            self.plist.set(names[i], values[i])
        f = open(self.fName,"w")
        self.plist.unused(f)
        f.close()
        lines = open(self.fName,"r").readlines()
        for i in range(len(lines)):
            self.assertEqual(lines[i],
                             'WARNING: Parameter "%s" %s   [unused] is unused\n' %
                             (names[i], values[i]))

    def testCurrentParametersString(self):
        "Test Teuchos.ParameterList currentParametersString method"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        names  = ["max its","tolerance"]
        values = [100      , 1e-6      ]
        types  = ["int"    ,"double"   ]
        result = "  {"
        for i in range(len(names)):
            self.plist.set(names[i], values[i])
            result += '\n    "%s" : %s = %g' % (names[i], types[i], values[i])
        result += "\n  }\n"
        self.assertEqual(self.plist.currentParametersString(), result)

    def testType(self):
        "Test Teuchos.ParameterList type method"
        sublist = Teuchos.ParameterList()
        names  = ["iParm", "fParm", "sParm", "lParm"              ]
        values = [2006   , 2.71828, "Hello", sublist              ]
        types  = [int    , float  , str    , Teuchos.ParameterList]
        for i in range(len(names)):
            self.plist.set(names[i],values[i])
        for i in range(len(names)):
            self.assertEqual(self.plist.type(names[i]), types[i])

    def test__cmp__1(self):
        "Test Teuchos.ParameterList __cmp__ method with dictionary"
        d1 = { "a" : 2 }
        d2 = { "a" : 2, "b" : 1 }
        plist = Teuchos.ParameterList(d1)
        self.assertEqual(cmp(plist, d1), 0)
        self.assertEqual(cmp(plist, d2),-1)

    def test__cmp__2(self):
        "Test Teuchos.ParameterList __cmp__ method with ParameterList"
        d = { "a" : 2, "b" : 1 }
        plist1 = Teuchos.ParameterList(d)
        plist2 = Teuchos.ParameterList(d)
        self.assertEqual(cmp(plist1, plist2), 0)
        plist1.set("c",3)
        self.assertEqual(cmp(plist1, plist2), 1)

    def test__contains__(self):
        "Test Teuchos.ParameterList __contains__ method"
        d = { "a" : 2, "b" : 1 }
        plist = Teuchos.ParameterList(d)
        self.assertEqual("b" in d, True )
        self.assertEqual("c" in d, False)

    def test__eq__1(self):
        "Test Teuchos.ParameterList __eq__ method with dictionary"
        d1 = { "a" : 2 }
        d2 = { "a" : 2, "b" : 1 }
        plist = Teuchos.ParameterList(d1)
        self.assertEqual(plist == d1, True )
        self.assertEqual(plist == d2, False)

    def test__eq__2(self):
        "Test Teuchos.ParameterList __eq__ method with ParameterList"
        d = { "a" : 2, "b" : 1 }
        pdplist = Teuchos.ParameterList(d)
        plist   = Teuchos.ParameterList(pdplist)
        self.assertEqual(pdplist == plist, True )
        plist.set("c",3)
        self.assertEqual(pdplist == plist, False)

    def test__getitem__(self):
        "Test Teuchos.ParameterList __getitem__ method"
        plist = Teuchos.ParameterList(self.dict)
        for key in plist.asDict():
            self.assertEqual(plist[key], plist.get(key))

    def test__getitem__bad1(self):
        "Test Teuchos.ParameterList __getitem__ method for nonexisting parameter"
        plist = Teuchos.ParameterList(self.dict)
        self.assertRaises(KeyError, plist.__getitem__, "a")

    def test__getitem__bad2(self):
        "Test Teuchos.ParameterList __getitem__ method for bad key"
        plist = Teuchos.ParameterList(self.dict)
        self.assertRaises(TypeError, plist.__getitem__, 20)

    def test__len__(self):
        "Test Teuchos.ParameterList __len__ method"
        plist = Teuchos.ParameterList(self.dict)
        self.assertEqual(len(plist), len(self.dict))

    def test__ne__1(self):
        "Test Teuchos.ParameterList __ne__ method with dictionary"
        d1 = { "a" : 2 }
        d2 = { "a" : 2, "b" : 1 }
        plist = Teuchos.ParameterList(d1)
        self.assertEqual(plist != d1, False)
        self.assertEqual(plist != d2, True )

    def test__ne__2(self):
        "Test Teuchos.ParameterList __ne__ method with ParameterList"
        d = { "a" : 2, "b" : 1 }
        plist1 = Teuchos.ParameterList(d)
        plist2 = Teuchos.ParameterList(plist1)
        self.assertEqual(plist1 != plist2, False)
        plist1.set("c",3)
        self.assertEqual(plist1 != plist2, True )

    def test__repr__(self):
        "Test Teuchos.ParameterList __repr__ method"
        plist = Teuchos.ParameterList(self.dict)
        d = eval("Teuchos." + repr(plist))
        self.assertEqual(isinstance(d, Teuchos.ParameterList), True)

    def test__setitem__1(self):
        "Test Teuchos.ParameterList __setitem__ method"
        self.plist[self.name] = 2006
        self.assertEqual(self.plist.get(self.name), 2006)

    def test__setitem__2(self):
        "Test Teuchos.ParameterList __setitem__ method for a dictionary"
        self.plist[self.name] = self.dict
        self.assertEqual(isinstance(self.plist.get(self.name),
                                    Teuchos.ParameterList), True)

    def test__setitem__3(self):
        "Test Teuchos.ParameterList __setitem__ method for a ParameterList"
        plist = Teuchos.ParameterList()
        self.plist[self.name] = plist
        self.assertEqual(isinstance(self.plist.get(self.name),
                                    Teuchos.ParameterList), True)

    def test__setitem__bad1(self):
        "Test Teuchos.ParameterList __setitem__ method for bad key"
        self.assertRaises(TypeError, self.plist.__setitem__, 0, "Hello")

    def test__setitem__bad2(self):
        "Test Teuchos.ParameterList __setitem__ method for bad value"
        self.assertRaises(TypeError, self.plist.__setitem__, "Hello", (9,4,1))

    def test__str__(self):
        "Test Teuchos.ParameterList __str__ method"
        # This does not work under MPI on my MacBook Pro yet...
        if self.commType == "MPI": return
        plist = Teuchos.ParameterList(self.dict)
        d = eval(str(plist))
        self.assertEqual(isinstance(d, dict), True)

    def testAsDict(self):
        "Test Teuchos.ParameterList asDict method"
        plist = Teuchos.ParameterList(self.dict)
        self.assertEqual(plist.asDict(), self.dict)

    def testItems(self):
        "Test Teuchos.ParameterList items method"
        plist = Teuchos.ParameterList(self.dict)
        self.assertEqual(plist.items(), self.dict.items())

    def testKeys(self):
        "Test Teuchos.ParameterList keys method"
        plist = Teuchos.ParameterList(self.dict)
        self.assertEqual(plist.keys(), self.dict.keys())

    def testValues(self):
        "Test Teuchos.ParameterList values method"
        plist = Teuchos.ParameterList(self.dict)
        self.assertEqual(plist.values(), self.dict.values())

    def testIteritems(self):
        "Test Teuchos.ParameterList iteritems method"
        self.plist.setParameters(self.dict)
        items = self.dict.items()
        for i in self.plist.iteritems():
            self.assertEqual(i in items, True)

    def testIterkeys(self):
        "Test Teuchos.ParameterList iterkeys method"
        self.plist.setParameters(self.dict)
        keys = self.dict.keys()
        for i in self.plist.iterkeys():
            self.assertEqual(i in keys, True)

    def testItervalues(self):
        "Test Teuchos.ParameterList itervalues method"
        self.plist.setParameters(self.dict)
        values = self.dict.values()
        for i in self.plist.itervalues():
            self.assertEqual(i in values, True)

    def testUpdate1(self):
        "Test Teuchos.ParameterList update method for a ParameterList"
        intName    = "int parameter"
        intValue   = 8
        floatName  = "float parameter"
        floatValue = 3.14
        self.plist.set(intName,  intValue  )
        self.plist.set(floatName,floatValue)
        newList = Teuchos.ParameterList()
        newList.update(self.plist)
        self.assertEqual(newList.get(intName  ), intValue  )
        self.assertEqual(newList.get(floatName), floatValue)

    def testUpdate2(self):
        "Test Teuchos.ParameterList update method for a dictionary"
        self.plist.update(self.dict)
        self.assertEqual(self.plist, self.dict)

    def testUpdateBad(self):
        "Test Teuchos.ParameterList update method for unsupported type"
        d = dict(i=5,f=10.0,t=(1,2,3,5,7))
        self.assertRaises(ValueError, self.plist.update, d)

    def test__iter__(self):
        "Test Teuchos.ParameterList __iter__ method"
        values = range(10)
        for i in values:
            self.plist.set(str(i), i)
        keys = self.plist.keys()
        for p in self.plist:        # This invokes the __iter__ method
            self.assertEqual(p in keys, True)

####################################################################

if __name__ == "__main__":

    # Create the test suite object
    suite = unittest.TestSuite()

    # Add the test cases to the test suite
    suite.addTest(unittest.makeSuite(ParameterListTestCase))

    # Create a communicator
    comm    = Teuchos.DefaultComm.getComm()
    iAmRoot = comm.getRank() == 0

    # Run the test suite
    if iAmRoot: print >>sys.stderr, \
       "\n*****************************\n" + \
       "Testing Teuchos.ParameterList\n" + \
       "*****************************\n"
    v = verbosity * int(iAmRoot)
    result = unittest.TextTestRunner(verbosity=v).run(suite)

    # Exit with a code that indicates the total number of errors and failures
    errsPlusFails = comm.reduceAll(Teuchos.REDUCE_SUM,
                                   len(result.errors) + len(result.failures))
    if errsPlusFails == 0 and iAmRoot: print "End Result: TEST PASSED"
    sys.exit(errsPlusFails)
